@@ -1,5 +1,6 @@
 #!/usr/bin/env python
-# Copyright (c) 2007 ActiveState Software Inc.
+# Copyright (c) 2007-2008 ActiveState Software Inc.
+# License: MIT (http://www.opensource.org/licenses/mit-license.php)
 
 """The markdown2 test suite entry point."""
 
@@ -25,7 +26,14 @@ def setup():
         sys.path.insert(0, pygments_dir)
 
 if __name__ == "__main__":
+    default_tags = ["-knownfailure"]
+    try:
+        import pygments
+    except ImportError:
+        # Skip pygments tests.
+        default_tags.append("-pygments")
     retval = testlib.harness(testdir_from_ns=testdir_from_ns,
-                             setup_func=setup)
+                             setup_func=setup,
+                             default_tags=default_tags)
     sys.exit(retval)
 
